function [ results ] = test4snr_sfft_algorithm_multipletests( d_array, N_array, signal_sparsity_k, algorithm_name, coeff_randtype, SNR_array, sparsity_s, sparsity_s_local, niter_r, niter_r_basic, nruns, filename )
%TEST4SNR_SFFT_ALGORITHM_MULTIPLETESTS Runs multiple test sets of sfft_algorithm.
%   This function may run many tests sets with a large amount of repititions.
%
%   Input parameters:
%      d_array - array of dimensions d
%      N_array - array of search space parameters N, Gamma = [-N,..,N]^d
%      signal_sparsity_k - signal sparsity
%      algorithm_name - type of algorithm,
%                       either 'a1r1l' for Algorithm 1
%                       or 'a2r1l' for Algorithm 2
%      coeff_randtype - type of random coefficients,
%                       either 'rand' for fhat_k in [-1,1] + 1i*[-1,1]
%                       with |fhat_k| >= 1e-6
%                       or 'phase' for fhat_k = exp(2*pi*1i*phi_k),
%                       phi_k in [0,1)
%      SNR_array - array of (linear) signal-to-noise ratios
%      sparsity_s - (optional) target sparsity of output, default (2*N+1)^d
%      sparsity_s_local - (optional) value of sparsity_s in local loops,
%                         uses the value of sparsity_s as default
%      niter_r - (optional) number of detection iteration r in steps 2b-2e,
%                default is 1
%      niter_r_basic - (optional) number of detection iteration r in
%                      steps 1 and 2a, uses the value of niter_r as default
%      nruns - number of repetition for one test setting
%      filename - output file name, where the output results are written to
%
%   Output parameter:
%      results - 4d array of type struct with size
%                [length(N_array), length(d_array), length(SNR_array), nruns]
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if nargin < 12
  filename = 'test4_results.mat';
end

results = repmat(struct(), length(N_array), length(d_array), length(SNR_array), nruns);

for iN=1:length(N_array)
  N = N_array(iN);
  for id=1:length(d_array)
    d = d_array(id);
    for inoise=1:length(SNR_array)
      SNR = SNR_array(inoise);
      for ir = 1:nruns
        [ num_freq, num_freq_out, num_intersect, max_err_abs, err_l2_rel, SNR_out, SNR_out2, noise_sigma, cand_sizes, lattice_sizes, sample_sizes, z, run_times ] = test4snr_sfft_algorithm_singletest( d, N, signal_sparsity_k, algorithm_name, coeff_randtype, SNR, sparsity_s, sparsity_s_local, niter_r, niter_r_basic );
fprintf('run=%2d, N=%3d, d=%2d, k=%6d, k_out=%6d, nfreq_both=%6d, err_abs=%.3e, err_l2_rel=%.3e, SNR_out=%.3e, noise_sigma=%.3e\n', ir, N, d, num_freq, num_freq_out, num_intersect, max_err_abs, err_l2_rel, SNR_out, noise_sigma);
drawnow;
        results(iN,id,inoise,ir).num_freq = num_freq;
        results(iN,id,inoise,ir).num_freq_out = num_freq_out;
        results(iN,id,inoise,ir).num_intersect = num_intersect;
        results(iN,id,inoise,ir).max_err_abs = max_err_abs;
        results(iN,id,inoise,ir).err_l2_rel = err_l2_rel;
        results(iN,id,inoise,ir).cand_sizes = cand_sizes;
        results(iN,id,inoise,ir).lattice_sizes = lattice_sizes;
        results(iN,id,inoise,ir).sample_sizes = sample_sizes;
        results(iN,id,inoise,ir).z = z;
        results(iN,id,inoise,ir).SNR_out = SNR_out;
        results(iN,id,inoise,ir).SNR_out2 = SNR_out2;
        results(iN,id,inoise,ir).noise_sigma = noise_sigma;
        results(iN,id,ir).run_times = run_times; % basic, lattice search, sampling, lfft, all
        save(filename, 'd_array', 'N_array', 'signal_sparsity_k', 'algorithm_name', 'coeff_randtype', 'SNR_array', 'sparsity_s', 'sparsity_s_local', 'niter_r', 'niter_r_basic', 'nruns', 'results');
      end
    end
  end
end

end

